@@ -20,7 +20,7 @@ describe('errorHandle', () => {
 		expect({ actual, ...errors }).toMatchSnapshot()
 	})
 
-	it('test', () => {
+	it('should encode < literal when not part of a tag', () => {
 		const description = '<p>populaciji (< 0.1%), te se</p>'
 		const { errors, parser } = getTestParser()
 
